
{*******************************************************}
{                                                       }
{       Animated Menus                                  }
{       TMainMenu2000 Component Unit                    }
{                                                       }
{       Copyright (c) 1997-2001 AnimatedMenus.com       }
{       All rights reserved.                            }
{                                                       }
{*******************************************************}


unit am2000mainmenu;

{$I am2000.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls, ExtCtrls,
  Forms, Dialogs, Menus, Buttons, ComCtrls,
  am2000options, am2000menuitem, am2000utils;

type

  // mainmenu manager
  TCustomMainMenu2000 = class(TMainMenu)
  private
    FEMI2000        : TEditableMenuItem2000; // layer for comptibility with Delphi and your apps
    FMergedMenus    : TList;
    FMergedMenuItems: TList;
    ParentMenu      : TCustomMainMenu2000;
    PopulatedMenus  : TList;

    FSBPanelNo      : Integer;
    FStatusBar      : TStatusBar;
    FOnMenuCommand  : TNotifyEvent;
    FOnMenuClose    : T_AM2000_MenuCloseEvent;
    FFont           : TFont;
    FParentFont     : Boolean;
    FParentShowHint : Boolean;
    FShowHint       : Boolean;
    FSystemFont     : Boolean;
    FOnCloseQuery   : TCloseQueryEvent;
    FOptions        : T_AM2000_Options;
    FCtl3D          : Boolean;

{$IFNDEF Delphi4OrHigher}
    FImages         : TImageList;
{$ENDIF}

    FHideProperty   : Integer;
    FOnContextMenu  : TNotifyEvent;

    function GetMenuItem2000: TMenuItem2000;
    function GetMergedMenuItem(const Index: Integer): TMenuItem;
    function GetMergedMenuItemsCount: Integer;
    function IsFontStored: Boolean;
    function IsShowHintStored: Boolean;
    procedure SetFont(Value: TFont);
    procedure SetParentFont(Value: Boolean);
    procedure SetShowHint(Value: Boolean);
    procedure SetParentShowHint(Value: Boolean);
    procedure SetSystemFont(Value: Boolean);
    procedure SetOptions(Value: T_AM2000_Options);
    procedure SetVersion(Value: T_AM2000_Version);
    function GetVersion: T_AM2000_Version;

    procedure cmParentFontChanged(var Msg: TMessage); message cm_ParentFontChanged;
    procedure cmFontChanged(var Msg: TMessage); message cm_FontChanged;

    procedure OnFontChange(Sender: TObject);

  protected

    property Version: T_AM2000_Version
      read GetVersion write SetVersion stored False;

    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
    Ole2ObjectWindow    : HWnd;
    Ole2MenuDescriptor  : HMenu;

    property Items2000 : TMenuItem2000
      read GetMenuItem2000;

    property MergedMenus : TList
      read FMergedMenus;

    property StatusBar       : TStatusBar
      read FStatusBar write FStatusBar;
    property StatusBarIndex  : Integer
      read FSBPanelNo write FSBPanelNo default 0;
    property ShowHint        : Boolean
      read FShowHint write SetShowHint stored IsShowHintStored;
    property Options         : T_AM2000_Options
      read FOptions write SetOptions;
    property Font            : TFont
      read FFont write SetFont stored IsFontStored;
    property SystemFont      : Boolean
      read FSystemFont write SetSystemFont default True;
    property Ctl3D           : Boolean
      read FCtl3D write FCtl3D default True;
    property ParentShowHint  : Boolean
      read FParentShowHint write SetParentShowHint default True;
    property ParentFont      : Boolean
      read FParentFont write SetParentFont default False;

    property OnMenuCommand   : TNotifyEvent
      read FOnMenuCommand write FOnMenuCommand;
    property OnMenuClose     : T_AM2000_MenuCloseEvent
      read FOnMenuClose write FOnMenuClose;
    property OnCloseQuery    : TCloseQueryEvent
      read FOnCloseQuery write FOnCloseQuery;
    property OnContextMenu   : TNotifyEvent
      read FOnContextMenu write FOnContextMenu;

{$IFNDEF Delphi4OrHigher}
    property Images : TImageList
      read FImages write FImages;
{$ENDIF}

    property MergedMenuItems [const Index: Integer] : TMenuItem read GetMergedMenuItem;
    property MergedMenuItemsCount : Integer read GetMergedMenuItemsCount;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Merge2000(Menu: TMenu);
    procedure Unmerge2000(Menu: TMenu);
    procedure Merge2(Menu: TMenu);
    procedure Unmerge2(Menu: TMenu); // backward-compatibility function
    procedure RebuildMergedMenuItems; // backward-compatibility function
    procedure Refresh;

    function IsShortCut(var Msg: TWMKey): Boolean;
{$IFDEF Delphi4OrHigher}
    override;
{$ENDIF}

//    function FindItem2000(Value: Word; Kind: TFindItemKind): TMenuItem;

    // Ole2 menu support
    procedure PopulateOle2Menu2000(hSharedMenu: HMenu; Groups: array of Integer;
      var Widths: array of Longint);
    procedure SetOle2MenuHandle2000(hMenuShared, hOleMenu: HMenu; hwndActiveObject: HWND);
//    function GetHandle: HMENU; override;
    function IsMenuPopulated(hMenu: HMenu): Boolean;

  published
    property Items       : TEditableMenuItem2000
      read FEMI2000;

    // hide these properties from ObjectInspector
    property AutoMerge: Integer read FHideProperty;
    property AutoHotKeys: Integer read FHideProperty;
    property AutoLineReduction: Integer read FHideProperty;
    property BiDiMode: Integer read FHideProperty;
    property OwnerDraw: Integer read FHideProperty;
    property ParentBiDiMode: Integer read FHideProperty;
    property Tag: Integer read FHideProperty;

  end;


{$IFDEF Delphi3OrHigher }
  function GetMainMenu2000(Form: TCustomForm): TCustomMainMenu2000;
{$ELSE}
  function GetMainMenu2000(Form: TForm): TCustomMainMenu2000;
{$ENDIF}

implementation

uses
  Consts,
  am2000menubar, am2000cache;

type

  // forgot about this
  TFalseMenu = class(TComponent)
  private
{$IFDEF Delphi4OrHigher}
    FBiDiMode: TBiDiMode;
{$ENDIF}
    FItems: TMenuItem;
  end;


{ GetMainMenu2000 }

{$IFDEF Delphi3OrHigher }
  function GetMainMenu2000(Form: TCustomForm): TCustomMainMenu2000;
{$ELSE}
  function GetMainMenu2000(Form: TForm): TCustomMainMenu2000;
{$ENDIF}
var
  MenuBar: TCustomMenuBar2000;
begin
  MenuBar:= GetActiveMenuBar;
  if (MenuBar <> nil)
  then
    if MenuBar.Menu is TCustomMainMenu2000
    then Result:= TCustomMainMenu2000(MenuBar.Menu)
    else raise EMenuError.Create(SOnlyMainMenuCanBeMerged)
  else
    Result:= nil;
end;


{ TCustomMainMenu2000 }

constructor TCustomMainMenu2000.Create(AOwner: TComponent);
begin
  inherited;

  FFont:= TFont.Create;
  FParentFont:= False;
  FParentShowHint:= True;
  FSystemFont:= True;
  FCtl3D:= True;
  FOptions:= T_AM2000_Options.Create;
  
  // we don't need TMenuItem so we just destoy it
  // and create TMenuItem2000 instead of.
  TFalseMenu(Self).FItems.Free;
  TFalseMenu(Self).FItems:= TMenuItem2000.Create(Self);
{$IFDEF Delphi4OrHigher}
  TFalseMenu(Self).FBiDiMode:= bdLeftToRight;
{$ENDIF}
  FEMI2000:= TEditableMenuItem2000.Create(Self);

  FMergedMenus:=  TList.Create;
  FMergedMenus.Add(Self);

  FMergedMenuItems:= TList.Create;

  InstallWindowsHooks;
end;

destructor TCustomMainMenu2000.Destroy;
begin
  RemoveWindowsHooks;

  if (ParentMenu <> nil)
  then ParentMenu.Unmerge2(Self);

  FFont.Free;
  FOptions.Free;
  PopulatedMenus.Free;
  FMergedMenus.Free;
  FMergedMenuItems.Free;

  PostMessage(GetMnuDsgnHandle, wm_Close, 0, 0);

  inherited;
end;

procedure TCustomMainMenu2000.Loaded;
var
  F: TForm;
  I: Integer;
  M: TCustomMenuBar2000;
begin
  inherited;

  if (not (csDesigning in ComponentState))
  and (Owner is TForm)
  then begin
    F:= TForm(Owner);

    // if no menu bars specified, create one for me
    if (F.Menu = Self)
    and (F.FormStyle <> fsMdiChild)
    and (GetMenuBarFromMenu(Self) = nil)
    then begin
      F.Menu:= nil;
      M:= TCustomMenuBar2000.Create(Owner);
      M.Align:= alTop;
      M.Parent:= F;
      M.Menu:= Self;
    end;


    // if this menu is the main menu for a mdi child form
    // then scan the parent mdi form and merge with
    // it's main menu
    if (F.Menu = Self)
    and (F.FormStyle = fsMdiChild)
    and (ActiveMenuBarList.Count > 0)
    then
      for I:= 0 to ActiveMenuBarList.Count -1
      do begin
        M:= TCustomMenuBar2000(ActiveMenuBarList[I]);
        if (M.OwnerForm <> nil)
        and (M.OwnerForm.FormStyle = fsMdiForm)
        and (M.Menu is TCustomMainMenu2000)
        then begin
          ParentMenu:= TCustomMainMenu2000(M.Menu);
          ParentMenu.Merge2000(Self);
          Exit;
        end;
      end;
  end;

  Font.OnChange:= OnFontChange;
end;

function TCustomMainMenu2000.GetMenuItem2000: TMenuItem2000;
begin
  Result:= TMenuItem2000(inherited Items);
end;

procedure TCustomMainMenu2000.Merge2(Menu: TMenu);
begin
end;

procedure TCustomMainMenu2000.Merge2000(Menu: TMenu);
begin
end;

procedure TCustomMainMenu2000.Unmerge2(Menu: TMenu);
begin
end;

procedure TCustomMainMenu2000.Unmerge2000(Menu: TMenu);
begin
end;

procedure TCustomMainMenu2000.RebuildMergedMenuItems;
var
  GroupIndex, MenuIndex, ItemIndex: Integer;
  BreakGroupIndex: Boolean;
  Menu: TMenu;
  OwnerForm: TComponent;
begin
  FMergedMenuItems.Clear;

  // place menu for active form at the end of the list
  for MenuIndex:= 0 to FMergedMenus.Count -1 do
    if FMergedMenus[MenuIndex] <> Self then begin
      Menu:= TMenu(FMergedMenus[MenuIndex]);
      OwnerForm:= Menu.Owner;
      while (OwnerForm <> nil)
      and (not (OwnerForm is TForm)) do
        OwnerForm:= OwnerForm.Owner;

      if (OwnerForm is TForm)
      and (TForm(OwnerForm).Active)
      and (TForm(OwnerForm).Menu = Menu)
      then begin
        FMergedMenus.Remove(Menu);
        FMergedMenus.Add(Menu);
        Break;
      end;
    end;

  // rebuild menu items
  for GroupIndex:= 0 to 255 do begin
    MenuIndex:= FMergedMenus.Count -1;
    BreakGroupIndex:= False;
    while (MenuIndex >= 0) and (not BreakGroupIndex) do
      with TMenu(FMergedMenus[MenuIndex]) do begin
        for ItemIndex:= 0 to Items.Count -1 do
          if Items[ItemIndex].GroupIndex = GroupIndex then begin
            FMergedMenuItems.Add(Items[ItemIndex]);
            BreakGroupIndex:= True;
          end;

        Dec(MenuIndex);
      end;
  end;
end;

function TCustomMainMenu2000.GetMergedMenuItem(const Index: Integer): TMenuItem;
begin
  Result:= nil;

  if (FMergedMenuItems = nil) or (Index < 0)
  then Exit;

  if FMergedMenuItems.Count = 0
  then RebuildMergedMenuItems;

  if (Index < FMergedMenuItems.Count)
  then Result:= TMenuItem(FMergedMenuItems[Index]);
end;

function TCustomMainMenu2000.GetMergedMenuItemsCount: Integer;
begin
  if FMergedMenuItems = nil
  then
    Result:= 0

  else begin
    if (FMergedMenuItems.Count = 0)
    then RebuildMergedMenuItems;

    Result:= FMergedMenuItems.Count;
  end;
end;

procedure TCustomMainMenu2000.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;

  if (Operation = opRemove)
  then begin
{$IFNDEF Delphi4OrHigher}
    if (AComponent = FImages) then FImages:= nil;
{$ENDIF}
    if (AComponent = FStatusBar) then FStatusBar:= nil;
  end;
end;

function TCustomMainMenu2000.IsShortCut(var Msg: TWMKey): Boolean;
begin
  Result:= IsShortCutEx(Msg, Items2000, csDesigning in ComponentState);
end;

//function TCustomMainMenu2000.FindItem2000(Value: Word; Kind: TFindItemKind): TMenuItem;
//begin
//  Result:= FindItem(Value, Kind);
//end;

{ Ole2 support }

procedure TCustomMainMenu2000.PopulateOle2Menu2000(hSharedMenu: HMenu; Groups: array of Integer;
      var Widths: array of Longint);
begin
end;

procedure TCustomMainMenu2000.SetOle2MenuHandle2000(hMenuShared, hOleMenu: HMenu; hwndActiveObject: HWND);
begin
end;

//function TCustomMainMenu2000.GetHandle: HMENU;
//begin
//  Result:= inherited GetHandle;
//end;

function TCustomMainMenu2000.IsMenuPopulated(hMenu: HMenu): Boolean;
begin
  Result:= False;
end;

procedure TCustomMainMenu2000.SetVersion(Value: T_AM2000_Version);
begin
end;

function TCustomMainMenu2000.GetVersion: T_AM2000_Version;
begin
  Result:= SVersion;
end;

function TCustomMainMenu2000.IsFontStored: Boolean;
begin
  Result:= not (FParentFont or FSystemFont);
end;

function TCustomMainMenu2000.IsShowHintStored: Boolean;
begin
  Result:= not FParentShowHint;
end;

procedure TCustomMainMenu2000.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
end;

procedure TCustomMainMenu2000.SetParentFont(Value: Boolean);
begin
  FParentFont:= Value;
  if Value then FSystemFont:= False;
end;

procedure TCustomMainMenu2000.SetSystemFont(Value: Boolean);
begin
  FSystemFont:= Value;
  if Value then FParentFont:= False;
end;

procedure TCustomMainMenu2000.SetShowHint(Value: Boolean);
begin
  if FShowHint <> Value then begin
    FShowHint:= Value;
    FParentShowHint:= False;
  end;
end;

procedure TCustomMainMenu2000.SetParentShowHint(Value: Boolean);
begin
  if FParentShowHint <> Value then
    FParentShowHint:= Value;

  if FParentShowHint then
    if Owner is TControl
    then FShowHint:= TControl(Owner).ShowHint
    else FShowHint:= False;

end;

procedure TCustomMainMenu2000.SetOptions(Value: T_AM2000_Options);
begin
  FOptions.Assign(Value);
end;

procedure TCustomMainMenu2000.cmFontChanged(var Msg: TMessage);
var
  M: TCustomMenuBar2000;
begin
  M:= GetMenuBarFromMenu(Self);
  if M <> nil then M.UpdateMenuBar(True);
end;

procedure TCustomMainMenu2000.cmParentFontChanged(var Msg: TMessage);
var
  M: TCustomMenuBar2000;
begin
  M:= GetMenuBarFromMenu(Self);
  if M <> nil then M.UpdateMenuBar(True);
end;

procedure TCustomMainMenu2000.OnFontChange(Sender: TObject);
var
  M: TCustomMenuBar2000;
begin
  M:= GetMenuBarFromMenu(Self);
  if M <> nil then M.UpdateMenuBar(True);
end;

procedure TCustomMainMenu2000.Refresh;
var
  M: TCustomMenuBar2000;
begin
  M:= GetMenuBarFromMenu(Self);
  if M <> nil
  then begin
    M.UpdateMenuBar(True);
    if M.MenuComponent <> nil
    then M.MenuComponent.Refresh;
  end;
end;



end.
